@@ -7,7 +7,7 @@ packet_names = ['OPEN', 'CLOSE', 'PING', 'PONG', 'MESSAGE', 'UPGRADE', 'NOOP']
 binary_types = (bytes, bytearray)
 
 
-class Packet(object):
+class Packet:
     """Engine.IO packet."""
 
     json = _json
@@ -15,6 +15,7 @@ class Packet(object):
     def __init__(self, packet_type=NOOP, data=None, encoded_packet=None):
         self.packet_type = packet_type
         self.data = data
+        self.encode_cache = None
         if isinstance(data, str):
             self.binary = False
         elif isinstance(data, binary_types):
@@ -27,7 +28,13 @@ class Packet(object):
             self.decode(encoded_packet)
 
     def encode(self, b64=False):
-        """Encode the packet for transmission."""
+        """Encode the packet for transmission.
+
+        Note: as a performance optimization, subsequent calls to this method
+        will return a cached encoded packet, even if the data has changed.
+        """
+        if self.encode_cache:
+            return self.encode_cache
         if self.binary:
             if b64:
                 encoded_packet = 'b' + base64.b64encode(self.data).decode(
@@ -43,11 +50,14 @@ class Packet(object):
                                                   separators=(',', ':'))
             elif self.data is not None:
                 encoded_packet += str(self.data)
+        self.encode_cache = encoded_packet
         return encoded_packet
 
     def decode(self, encoded_packet):
         """Decode a transmitted package."""
         self.binary = isinstance(encoded_packet, binary_types)
+        if not self.binary and len(encoded_packet) == 0:
+            raise ValueError('Invalid empty packet received')
         b64 = not self.binary and encoded_packet[0] == 'b'
         if b64:
             self.binary = True
